﻿using Connection.Class;
using Microsoft.Win32;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using WBP_O3.Class;

namespace WBP_O3_API_Demo_Customer {

    /// <summary>
    /// MainWindow.xaml 的互動邏輯
    /// </summary>
    public partial class MainWindow : Window {

        public MainWindow() {
            InitializeComponent();
            System.Reflection.Assembly assembly = System.Reflection.Assembly.GetExecutingAssembly();
            var WBP_O3Info = System.Reflection.Assembly.GetAssembly(typeof(WBP_O3.WBP_O3)).GetName();
            Title = $"{assembly.GetName().Name} v{assembly.GetName().Version} {WBP_O3Info.Name}  v{WBP_O3Info.Version}";
        }

        private static WBP_O3.WBP_O3 o3;
        private string key = "";

        private void Form1_Load(object sender, EventArgs e) {
            Command_comboBox.SelectedIndex = 0;
            DataFormat_comboBox.SelectedIndex = 1;
        }

        private void O3_OnDisConnected() {
            try {
                Dispatcher.Invoke(() => {
                    Label_DevicePath.Content = "";
                    button_Send.IsEnabled = false;
                    this.Sch_2_BPM_Setting_GroupBOX.Visibility = Visibility.Hidden;
                    this.Sch_5_BPM_Setting_GroupBOX.Visibility = Visibility.Hidden;
                });
            } catch { }
        }

        private void O3_OnConnected() {
            Dispatcher.Invoke(() => {
                Label_DevicePath.Content = o3.DevicePath;
                button_Send.IsEnabled = true;
            });
        }

        private async void Button_SendCommand_Click(object sender, EventArgs e) {
            RawData_TextBox.SetText("");
            (bool Success, Command Command) callback;
            StringBuilder sb = new StringBuilder();
            switch (Command_comboBox.SelectedIndex) {
                case 0: //Read all history data from BPM
                    callback = await o3.SendMessageAsync(WBP_O3.SendMessage.ToCommand.dataReadAll());
                    if (callback.Command.CMD == (byte)WBP_O3.Enum.Command.NACK) {
                        sb.Append("NACK");
                        return;
                    } else if (!callback.Success) {
                        sb.Append("No call back,Please try again");
                        return;
                    }
                    Index_comboBox.Items.Clear();
                    List<MData> listMData = (List<MData>)callback.Command.Data;
                    listMData.Where(x => x.Condition == 1 && x.CSBP > 0).ToList().ForEach(x => Index_comboBox.Items.Add(x.Index));
                    try {
                        Index_comboBox.SelectedIndex = 0;
                    } catch { }
                    Connection.GlobalFunction.PrintAllProperties(listMData, ref sb);
                    break;

                case 1: //Read central BP memory data by index from BPM
                    if (Index_comboBox.SelectedIndex == -1) {
                        sb.Append("Select a id in combobox first.");
                        return;
                    }
                    callback = o3.SendMessage(WBP_O3.SendMessage.ToCommand.dataRead((int)Index_comboBox.SelectedItem, (WBP_O3.Enum.DataFormat)(DataFormat_comboBox.SelectedIndex == 2 ? 3 : DataFormat_comboBox.SelectedIndex)));
                    if (callback.Command.CMD == (byte)WBP_O3.Enum.Command.NACK) {
                        sb.Append("NACK");
                        return;
                    } else if (!callback.Success) {
                        sb.Append("No call back,Please try again");
                        return;
                    }
                    Data data = (Data)callback.Command.Data;
                    Connection.GlobalFunction.PrintAllProperties(data, ref sb);
                    break;

                case 2: //Clear the all history data of the BPM
                    callback = o3.SendMessage(WBP_O3.SendMessage.ToCommand.DataClear());
                    if (callback.Command.CMD == (byte)WBP_O3.Enum.Command.NACK) {
                        sb.Append("NACK");
                        return;
                    } else if (!callback.Success) {
                        sb.Append("No call back,Please try again");
                        return;
                    }
                    sb.Append("Memory have clear!");
                    break;

                case 3: //Read user ID and version data from BPM.
                    callback = o3.SendMessage(WBP_O3.SendMessage.ToCommand.userInfoRead());
                    if (callback.Command.CMD == (byte)WBP_O3.Enum.Command.NACK) {
                        sb.Append("NACK");
                        return;
                    } else if (!callback.Success) {
                        sb.Append("No call back,Please try again");
                        return;
                    }
                    UserInfo userInfo = (UserInfo)callback.Command.Data;
                    sb.AppendFormat("UserID:{0}", userInfo.UserID).AppendLine();
                    sb.AppendFormat("Protocal ID:{0}", userInfo.ProtocolID).AppendLine();
                    sb.AppendFormat("Max memory:{0}", userInfo.MaxMemory).AppendLine();
                    sb.AppendFormat("IHB:{0}", userInfo.IHB).AppendLine();
                    sb.AppendFormat("Firewall version:{0}", userInfo.FMVersion).AppendLine();
                    sb.AppendFormat("Firewall Date:{0}", userInfo.FMDate.ToString("yyyy-MM-dd")).AppendLine();
                    sb.AppendFormat("CBP:{0}", userInfo.CBP).AppendLine();
                    sb.AppendFormat("Voltage:{0} V", userInfo.BatteryVoltage).AppendLine();
                    sb.AppendFormat("arrhythmiaDisplay:{0}", userInfo.arrhythmiaDisplay.ToString()).AppendLine();
                    sb.AppendFormat("AFIB:{0}", userInfo.Afib).AppendLine();
                    textBox_NewID.Text = userInfo.UserID;
                    //Check Protocol ID , if less than V1.05 means 2 period ABPM
                    if (Convert.ToInt32(userInfo.ProtocolID.Replace("V","").Replace(".","")) < 105)
                        this.Sch_2_BPM_Setting_GroupBOX.Visibility = Visibility.Visible;
                    else
                        this.Sch_5_BPM_Setting_GroupBOX.Visibility = Visibility.Visible;
                    
                    break;

                case 4: //Write a new user ID to BPM.
                    callback = o3.SendMessage(WBP_O3.SendMessage.ToCommand.userIDWrite(textBox_NewID.Text));
                    if (callback.Command.CMD == (byte)WBP_O3.Enum.Command.NACK) {
                        sb.Append("NACK");
                        return;
                    } else if (!callback.Success) {
                        sb.Append("No call back,Please try again");
                        return;
                    }
                    sb.Append("The new ID was written successfully");
                    break;

                case 5: //Read Two Schedule Setting
                    callback = o3.SendMessage(WBP_O3.SendMessage.ToCommand.settingRead_ABPM());
                    if (callback.Command.CMD == (byte)WBP_O3.Enum.Command.NACK) {
                        sb.Append("NACK");
                        return;
                    } else if (!callback.Success) {
                        sb.Append("No call back,Please try again");
                        return;
                    }
                    Setting_ABPM setting_ABPM = (Setting_ABPM)callback.Command.Data;
                    Connection.GlobalFunction.PrintAllProperties(setting_ABPM, ref sb);
                    ABPMStart_1_textBox.Text = setting_ABPM.ABPMStart_1.ToString();
                    ABPMEnd_1_textBox.Text = setting_ABPM.ABPMEnd_1.ToString();
                    ABPMInt_1_textBox.Text = setting_ABPM.ABPMInt_1.ToString();
                    ABPMInt_2_textBox.Text = setting_ABPM.ABPMInt_2.ToString();
                    CBP_zone1_meas_off_comboBox.SelectedIndex = setting_ABPM.CBP_zone1_meas_off ? 1 : 0;
                    CBP_zone2_meas_off_comboBox.SelectedIndex = setting_ABPM.CBP_zone2_meas_off ? 1 : 0;
                    CBPInt_1_textBox.Text = setting_ABPM.CBPInt_1.ToString();
                    CBPInt_2_textBox.Text = setting_ABPM.CBPInt_2.ToString();
                    HI_infPressure_textBox.Text = setting_ABPM.HI_infPressure.ToString();
                    SW_checkhide_comboBox.SelectedIndex = setting_ABPM.SW_checkhide ? 1 : 0;
                    SW_SEL_silent_comboBox.SelectedIndex = setting_ABPM.SW_SEL_silent ? 1 : 0;
                    
                    break;
                    
                case 6: //Write Two Schedule Setting
                    Setting_ABPM Newsetting_ABPM = new Setting_ABPM() {
                        ABPMStart_1 = int.Parse(ABPMStart_1_textBox.Text),
                        ABPMEnd_1 = int.Parse(ABPMEnd_1_textBox.Text),
                        ABPMInt_1 = int.Parse(ABPMInt_1_textBox.Text),
                        ABPMInt_2 = int.Parse(ABPMInt_2_textBox.Text),
                        CBP_zone1_meas_off = CBP_zone1_meas_off_comboBox.SelectedIndex == 1,
                        CBP_zone2_meas_off = CBP_zone2_meas_off_comboBox.SelectedIndex == 1,
                        CBPInt_1 = int.Parse(CBPInt_1_textBox.Text),
                        CBPInt_2 = int.Parse(CBPInt_2_textBox.Text),
                        HI_infPressure = int.Parse(HI_infPressure_textBox.Text),
                        SW_checkhide = SW_checkhide_comboBox.SelectedIndex == 1,
                        SW_SEL_silent = SW_SEL_silent_comboBox.SelectedIndex == 1,
                    };
                    sb.Append("ABPM setting had been set to:\r\n");
                    Connection.GlobalFunction.PrintAllProperties(Newsetting_ABPM, ref sb);
                    callback = o3.SendMessage(WBP_O3.SendMessage.ToCommand.settingWrite_ABPM(Newsetting_ABPM));
                    if (callback.Command.CMD == (byte)WBP_O3.Enum.Command.NACK) {
                        sb.Append("NACK");
                        return;
                    } else if (!callback.Success) {
                        sb.Append("No call back,Please try again");
                        return;
                    }
                    sb.Append("Write Success!");
                    
                    break;

                case 7: //Read device ID and info from BPM.
                    callback = o3.SendMessage(WBP_O3.SendMessage.ToCommand.deviceIDRead());
                    if (callback.Command.CMD == (byte)WBP_O3.Enum.Command.NACK) {
                        sb.Append("NACK");
                        return;
                    } else if (!callback.Success) {
                        sb.Append("No call back,Please try again");
                        return;
                    }
                    DeviceID_and_Info deviceID_And_Info = (DeviceID_and_Info)callback.Command.Data;
                    Connection.GlobalFunction.PrintAllProperties(deviceID_And_Info, ref sb);
                    break;

                case 8: //Read device Time from BPM
                    callback = o3.SendMessage(WBP_O3.SendMessage.ToCommand.timeRead());
                    if (callback.Command.CMD == (byte)WBP_O3.Enum.Command.NACK) {
                        sb.Append("NACK");
                        return;
                    } else if (!callback.Success) {
                        sb.Append("No call back,Please try again");
                        return;
                    }
                    DateTime dateTime = (DateTime)callback.Command.Data;
                    sb.Append(dateTime.ToString("yyyy-MM-dd HH:mm:ss"));
                    break;

                case 9: //Write device Time to BPM
                    callback = o3.SendMessage(WBP_O3.SendMessage.ToCommand.timeWrite(DateTime.Now));
                    if (callback.Command.CMD == (byte)WBP_O3.Enum.Command.NACK) {
                        sb.Append("NACK");
                        return;
                    } else if (!callback.Success) {
                        sb.Append("No call back,Please try again");
                        return;
                    }
                    sb.Append("Time had been sync");
                    break;

                case 10:    //Read BPM function setting value from BPM.
                    callback = o3.SendMessage(WBP_O3.SendMessage.ToCommand.settingRead_BPM_Function());
                    if (callback.Command.CMD == (byte)WBP_O3.Enum.Command.NACK) {
                        sb.Append("NACK");
                        return;
                    } else if (!callback.Success) {
                        sb.Append("No call back,Please try again");
                        return;
                    }
                    Setting_BPM_Function setting_BPM = (Setting_BPM_Function)callback.Command.Data;
                    Connection.GlobalFunction.PrintAllProperties(setting_BPM, ref sb);
                    break;

                case 11:    //Read BTmodule name from BPM.
                    callback = o3.SendMessage(WBP_O3.SendMessage.ToCommand.bluetoothnameRead());
                    if (callback.Command.CMD == (byte)WBP_O3.Enum.Command.NACK) {
                        sb.Append("NACK");
                        return;
                    } else if (!callback.Success) {
                        sb.Append("No call back,Please try again");
                        return;
                    }
                    string s = (string)callback.Command.Data;
                    sb.Append(s);
                    break;

                case 12:    //Read SN
                    callback = o3.SendMessage(WBP_O3.SendMessage.ToCommand.deviceSNRead());
                    if (callback.Command.CMD == (byte)WBP_O3.Enum.Command.NACK) {
                        sb.Append("NACK");
                        return;
                    } else if (!callback.Success) {
                        sb.Append("No call back,Please try again");
                        return;
                    }
                    s = (string)callback.Command.Data;
                    sb.Append("Device SN:");
                    sb.Append(s);
                    textBox_NewSN.Text = s;
                    break;

                case 13:    //Read FiveSchedule Setting
                    callback = o3.SendMessage(WBP_O3.SendMessage.ToCommand.settingRead_FiveSchedule());
                    if (callback.Command.CMD == (byte)WBP_O3.Enum.Command.NACK) {
                        sb.Append("NACK");
                        return;
                    } else if (!callback.Success) {
                        sb.Append("No call back,Please try again");
                        return;
                    }
                    Connection.GlobalFunction.PrintAllProperties(callback.Command.Data, ref sb);
                    WBP_O3.ABPM_5ScheduleObj ABPM_Obj = (WBP_O3.ABPM_5ScheduleObj)callback.Command.Data;
                    HI_infPressureBox.Text = ABPM_Obj.HI_infPressure.ToString();
                    SW_checkhideCombobox.SelectedIndex = BooltoSelectIndex(ABPM_Obj.SW_checkhide);

                    StartBox1.Text = ABPM_Obj.ABPM_period_1.start.ToString();
                    EndBox1.Text = ABPM_Obj.ABPM_period_1.end.ToString();
                    IntervalBox1.Text = ABPM_Obj.ABPM_period_1.interval.ToString();
                    BP_off_comboBox1.SelectedIndex = BooltoSelectIndex(ABPM_Obj.ABPM_period_1.BP_meas_off);
                    CBP_off_comboBox1.SelectedIndex = BooltoSelectIndex(ABPM_Obj.ABPM_period_1.CBP_meas_off);
                    SW_Silent_comboBox1.SelectedIndex = BooltoSelectIndex(ABPM_Obj.ABPM_period_1.silent);
                    if (!ABPM_Obj.ABPM_period_1.disable)
                        EnablecomboBox1.SelectedIndex = 0;

                    StartBox2.Text = ABPM_Obj.ABPM_period_2.start.ToString();
                    EndBox2.Text = ABPM_Obj.ABPM_period_2.end.ToString();
                    IntervalBox2.Text = ABPM_Obj.ABPM_period_2.interval.ToString();
                    BP_off_comboBox2.SelectedIndex = BooltoSelectIndex(ABPM_Obj.ABPM_period_2.BP_meas_off);
                    CBP_off_comboBox2.SelectedIndex = BooltoSelectIndex(ABPM_Obj.ABPM_period_2.CBP_meas_off);
                    SW_Silent_comboBox2.SelectedIndex = BooltoSelectIndex(ABPM_Obj.ABPM_period_2.silent);
                    if (!ABPM_Obj.ABPM_period_2.disable)
                        EnablecomboBox2.SelectedIndex = 0;

                    StartBox3.Text = ABPM_Obj.ABPM_period_3.start.ToString();
                    EndBox3.Text = ABPM_Obj.ABPM_period_3.end.ToString();
                    IntervalBox3.Text = ABPM_Obj.ABPM_period_3.interval.ToString();
                    BP_off_comboBox3.SelectedIndex = BooltoSelectIndex(ABPM_Obj.ABPM_period_3.BP_meas_off);
                    CBP_off_comboBox3.SelectedIndex = BooltoSelectIndex(ABPM_Obj.ABPM_period_3.CBP_meas_off);
                    SW_Silent_comboBox3.SelectedIndex = BooltoSelectIndex(ABPM_Obj.ABPM_period_3.silent);
                    if (!ABPM_Obj.ABPM_period_3.disable)
                        EnablecomboBox3.SelectedIndex = 0;

                    StartBox4.Text = ABPM_Obj.ABPM_period_4.start.ToString();
                    EndBox4.Text = ABPM_Obj.ABPM_period_4.end.ToString();
                    IntervalBox4.Text = ABPM_Obj.ABPM_period_4.interval.ToString();
                    BP_off_comboBox4.SelectedIndex = BooltoSelectIndex(ABPM_Obj.ABPM_period_4.BP_meas_off);
                    CBP_off_comboBox4.SelectedIndex = BooltoSelectIndex(ABPM_Obj.ABPM_period_4.CBP_meas_off);
                    SW_Silent_comboBox4.SelectedIndex = BooltoSelectIndex(ABPM_Obj.ABPM_period_4.silent);
                    if (!ABPM_Obj.ABPM_period_4.disable)
                        EnablecomboBox4.SelectedIndex = 0;

                    StartBox5.Text = ABPM_Obj.ABPM_period_5.start.ToString();
                    EndBox5.Text = ABPM_Obj.ABPM_period_5.end.ToString();
                    IntervalBox5.Text = ABPM_Obj.ABPM_period_5.interval.ToString();
                    BP_off_comboBox5.SelectedIndex = BooltoSelectIndex(ABPM_Obj.ABPM_period_5.BP_meas_off);
                    CBP_off_comboBox5.SelectedIndex = BooltoSelectIndex(ABPM_Obj.ABPM_period_5.CBP_meas_off);
                    SW_Silent_comboBox5.SelectedIndex = BooltoSelectIndex(ABPM_Obj.ABPM_period_5.silent);
                    if (!ABPM_Obj.ABPM_period_5.disable)
                        EnablecomboBox5.SelectedIndex = 0;

                    break;
                case 14:    //Write FiveSchedule Setting
                    WBP_O3.ABPM_5ScheduleObj ABPM_5SchObj = new WBP_O3.ABPM_5ScheduleObj();
                    ABPM_5SchObj.HI_infPressure = StringToInt(HI_infPressureBox.Text);
                    if (SW_checkhideCombobox.SelectedIndex == 0)
                        ABPM_5SchObj.SW_checkhide = false;
                    else
                        ABPM_5SchObj.SW_checkhide = true;

                    ABPM_5SchObj.ABPM_period_1.start = StringToInt(StartBox1.Text);
                    ABPM_5SchObj.ABPM_period_1.end = StringToInt(EndBox1.Text);
                    ABPM_5SchObj.ABPM_period_1.interval = StringToInt(IntervalBox1.Text);
                    ABPM_5SchObj.ABPM_period_1.BP_meas_off = StringToBool(BP_off_comboBox1.SelectedItem.ToString());
                    ABPM_5SchObj.ABPM_period_1.CBP_meas_off = StringToBool(CBP_off_comboBox1.SelectedItem.ToString());
                    ABPM_5SchObj.ABPM_period_1.silent = ComboToBool(SW_Silent_comboBox1.SelectedIndex);
                    if (EnablecomboBox1.SelectedIndex == 0)
                        ABPM_5SchObj.ABPM_period_1.disable = false;
                    else
                        ABPM_5SchObj.ABPM_period_1.disable = true;

                    ABPM_5SchObj.ABPM_period_2.start = StringToInt(StartBox2.Text);
                    ABPM_5SchObj.ABPM_period_2.end = StringToInt(EndBox2.Text);
                    ABPM_5SchObj.ABPM_period_2.interval = StringToInt(IntervalBox2.Text);
                    ABPM_5SchObj.ABPM_period_2.BP_meas_off = StringToBool(BP_off_comboBox2.SelectedItem.ToString());
                    ABPM_5SchObj.ABPM_period_2.CBP_meas_off = StringToBool(CBP_off_comboBox2.SelectedItem.ToString());
                    ABPM_5SchObj.ABPM_period_2.silent = ComboToBool(SW_Silent_comboBox2.SelectedIndex);
                    if (EnablecomboBox2.SelectedIndex == 0)
                        ABPM_5SchObj.ABPM_period_2.disable = false;
                    else
                        ABPM_5SchObj.ABPM_period_2.disable = true;

                    ABPM_5SchObj.ABPM_period_3.start = StringToInt(StartBox3.Text);
                    ABPM_5SchObj.ABPM_period_3.end = StringToInt(EndBox3.Text);
                    ABPM_5SchObj.ABPM_period_3.interval = StringToInt(IntervalBox3.Text);
                    ABPM_5SchObj.ABPM_period_3.BP_meas_off = StringToBool(BP_off_comboBox3.SelectedItem.ToString());
                    ABPM_5SchObj.ABPM_period_3.CBP_meas_off = StringToBool(CBP_off_comboBox3.SelectedItem.ToString());
                    ABPM_5SchObj.ABPM_period_3.silent = ComboToBool(SW_Silent_comboBox3.SelectedIndex);
                    if (EnablecomboBox3.SelectedIndex == 0)
                        ABPM_5SchObj.ABPM_period_3.disable = false;
                    else
                        ABPM_5SchObj.ABPM_period_3.disable = true;

                    ABPM_5SchObj.ABPM_period_4.start = StringToInt(StartBox4.Text);
                    ABPM_5SchObj.ABPM_period_4.end = StringToInt(EndBox4.Text);
                    ABPM_5SchObj.ABPM_period_4.interval = StringToInt(IntervalBox4.Text);
                    ABPM_5SchObj.ABPM_period_4.BP_meas_off = StringToBool(BP_off_comboBox4.SelectedItem.ToString());
                    ABPM_5SchObj.ABPM_period_4.CBP_meas_off = StringToBool(CBP_off_comboBox4.SelectedItem.ToString());
                    ABPM_5SchObj.ABPM_period_4.silent = ComboToBool(SW_Silent_comboBox4.SelectedIndex);
                    if (EnablecomboBox4.SelectedIndex == 0)
                        ABPM_5SchObj.ABPM_period_4.disable = false;
                    else
                        ABPM_5SchObj.ABPM_period_4.disable = true;

                    ABPM_5SchObj.ABPM_period_5.start = StringToInt(StartBox5.Text);
                    ABPM_5SchObj.ABPM_period_5.end = StringToInt(EndBox5.Text);
                    ABPM_5SchObj.ABPM_period_5.interval = StringToInt(IntervalBox5.Text);
                    ABPM_5SchObj.ABPM_period_5.BP_meas_off = StringToBool(BP_off_comboBox5.SelectedItem.ToString());
                    ABPM_5SchObj.ABPM_period_5.CBP_meas_off = StringToBool(CBP_off_comboBox5.SelectedItem.ToString());
                    ABPM_5SchObj.ABPM_period_5.silent = ComboToBool(SW_Silent_comboBox5.SelectedIndex);
                    if (EnablecomboBox5.SelectedIndex == 0)
                        ABPM_5SchObj.ABPM_period_5.disable = false;
                    else
                        ABPM_5SchObj.ABPM_period_5.disable = true;

                    callback = o3.SendMessage(WBP_O3.SendMessage.ToCommand.settingWrite_FiveSchedule(ABPM_5SchObj));
                    if (callback.Command.CMD == (byte)WBP_O3.Enum.Command.NACK) {
                        sb.Append("NACK");
                        return;
                    } else if (!callback.Success) {
                        sb.Append("No call back,Please try again");
                        return;
                    }
                    sb.Append("The new setting was written successfully");
                    break;

                case 15:
                    o3.SendMessage(WBP_O3.SendMessage.ToCommand.bluetoothDisconnect());
                    break;
            }
            richTextBox1.SetText(sb.ToString());
        }

        bool StringToBool(string value) {
            if (value.Contains("False") || value.Contains("false") || value.Equals(""))
                return false;
            else
                return true;
        }
        bool ComboToBool(int value) {
            if (value == 0)
                return false;
            else
                return true;
        }
        int StringToInt(string value) {
            if (value.Equals(""))
                return 0;
            else
                return int.Parse(value);
        }
        int BooltoSelectIndex(bool value) {
            if (value == false)
                return 0;
            else
                return 1;

        }

        private void CheckKey_Click(object sender, EventArgs e) {
            try {
                key = textBox_key.Text;
                o3 = new WBP_O3.WBP_O3(key);
                o3.OnConnected += O3_OnConnected;
                o3.OnDisConnected += O3_OnDisConnected;
                o3.OnReceivedRaw += O3_OnReceivedRaw;
                o3.OnSendRaw += O3_OnSendRaw;
                o3.InitDeviceWatcher();
                button_BLE.IsEnabled = true;
                textBox_key.IsEnabled = false;
                (sender as Button).IsEnabled = false;
            } catch (Exception ee) {
                MessageBox.Show(ee.Message);
            }
        }

        private void O3_OnSendRaw(string EffectiveByte) {
            Dispatcher.InvokeAsync(() => {
                StringBuilder sb = new StringBuilder();
                sb.Append(RawData_TextBox.GetText());
                sb.Append(EffectiveByte);
                RawData_TextBox.SetText(sb.ToString());
            });
        }

        private void O3_OnReceivedRaw(byte[] EffectiveByte) {
            Dispatcher.InvokeAsync(() => {
            StringBuilder sb = new StringBuilder();
            sb.Append(RawData_TextBox.GetText());
            sb.Append("read:" + ToHexString(EffectiveByte));
            RawData_TextBox.SetText(sb.ToString());
            });
        }
        public static string ToHexString(byte[] bytes) // 0xae00cf => "AE00CF "
{
            string hexString = string.Empty;
            if (bytes != null) {
                StringBuilder strB = new StringBuilder();
                for (int i = 0; i < bytes.Length; i++) {
                    strB.Append(bytes[i].ToString("X2"));
                    strB.Append(" ");
                }
                hexString = strB.ToString();
            }
            return hexString;
        }

        [STAThread]
        private async void Button_BLE_Click(object sender, EventArgs e) {
            if (!Connection.GlobalFunction.WinRT_API_Support) {
                MessageBox.Show("Operating system requires Windows 10 1803 or higher!");
                return;
            }
            Window_BluetoothDiscover form_BLE_Discover = new Window_BluetoothDiscover();

            form_BLE_Discover.ShowDialog();
            switch (form_BLE_Discover.BLEAction) {
                case BLEAction.Connect:
                    o3?.Dispose();
                    o3 = new WBP_O3.WBP_O3(key, true);
                    o3.OnReceivedRaw += O3_OnReceivedRaw;
                    o3.OnSendRaw += O3_OnSendRaw;
                    o3.OnDisConnected += O3_OnDisConnected;
                    var connectionResult = await o3.ConnectWithBLE(form_BLE_Discover.BLE_ID,true);
                    if (connectionResult.ErrorMessage == "Device is unreachable (i.e. out of range or shutoff)") {
                        MessageBox.Show("Please check the connection or unpair BLE, both of PC and blood pressure monitor.");
                        O3_OnDisConnected();
                    }
                    if (o3.Connected) {
                        await Connection.PairingHelper.PairDeviceAsync(form_BLE_Discover.BLE_ID);
                        await Task.Delay(500);
                        try {
                            await o3.SendMessageAsync(WBP_O3.SendMessage.ToCommand.timeRead());
                        } catch (Exception ee) {
                            if (ee.Message.Contains("0x80650005")) {
                                //PC unpair BLE
                                await Connection.PairingHelper.UnpairDeviceAsync(form_BLE_Discover.BLE_ID);
                                MessageBox.Show("Please unpair BLE on blood pressure monitor.");
                                O3_OnDisConnected();
                                return;
                            }
                        }
                        O3_OnConnected();
                    }
                    break;

                case BLEAction.Disconnect:
                    if (o3.ConnectionType == 1) {
                        o3?.Dispose();
                        o3 = new WBP_O3.WBP_O3(key);
                        o3.OnConnected += O3_OnConnected;
                        o3.OnDisConnected += O3_OnDisConnected;
                    }
                    break;

                case BLEAction.Pair:
                    await Connection.PairingHelper.PairDeviceAsync(form_BLE_Discover.BLE_ID);
                    break;

                case BLEAction.Unpair:
                    await Connection.PairingHelper.UnpairDeviceAsync(form_BLE_Discover.BLE_ID);
                    break;
            }
        }

        private void Save_Button_Click(object sender, RoutedEventArgs e) {
            string fileText = richTextBox1.GetText();

            SaveFileDialog dialog = new SaveFileDialog() {               
                Filter = "Text Files(*.txt)|*.txt|All(*.*)|*"
            };
            dialog.FileName = DateTime.Now.ToString("yyyy-MM-dd-HH-mm-ss");
            if (dialog.ShowDialog() == true) {
                File.WriteAllText(dialog.FileName, fileText);
            }
        }

    }
}